package cn.iocoder.yudao.module.customer.service.customerCompetitor;

import java.util.*;
import javax.annotation.Resource;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerCompetitor.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCompetitor.CustomerCompetitorDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerCompetitor.CustomerCompetitorConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerCompetitor.CustomerCompetitorMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户竞争对手 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerCompetitorServiceImpl extends AbstractService<CustomerCompetitorMapper, CustomerCompetitorDO> implements CustomerCompetitorService {

    @Resource
    private CustomerCompetitorMapper competitorMapper;

    @Override
    public CustomerCompetitorDO createCompetitor(CustomerCompetitorCreateReqVO createReqVO) {

        // 校验存在
        Long count = competitorMapper.selectCount("name", createReqVO.getName());
        if (count != 0) {
            throw exception(COMPETITOR_EXISTS);
        }

        // 插入
        CustomerCompetitorDO competitor = CustomerCompetitorConvert.INSTANCE.convert(createReqVO);
        competitorMapper.insert(competitor);
        // 返回
        return competitor;
    }

    @Override
    public void updateCompetitor(CustomerCompetitorUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCompetitorExists(updateReqVO.getId());
        // 更新
        CustomerCompetitorDO updateObj = CustomerCompetitorConvert.INSTANCE.convert(updateReqVO);
        competitorMapper.updateById(updateObj);
    }

    @Override
    public void deleteCompetitor(Long id) {
        // 校验存在
        this.validateCompetitorExists(id);
        // 删除
        competitorMapper.deleteById(id);
    }

    private void validateCompetitorExists(Long id) {
        if (competitorMapper.selectById(id) == null) {
            throw exception(COMPETITOR_NOT_EXISTS);
        }
    }

    @Override
    public CustomerCompetitorDO getCompetitor(Long id) {
        return competitorMapper.selectById(id);
    }

    @Override
    public List<CustomerCompetitorDO> getCompetitorList(Collection<Long> ids) {
        return competitorMapper.selectBatchIds(ids);
    }

    @Override
    public List<CustomerCompetitorDO> getCompetitorListBySearchKey(String searchKey) {
        return competitorMapper.selectList(new LambdaQueryWrapper<CustomerCompetitorDO>()
                .like(StringUtils.isNotBlank(searchKey), CustomerCompetitorDO::getName, searchKey)
                .orderDesc(CustomerCompetitorDO::getId)
                .last("limit 30"));
    }

    @Override
    public PageResult<CustomerCompetitorDO> getCompetitorPage(CustomerCompetitorQueryVO query, PageVO page) {
        return competitorMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerCompetitorDO> getCompetitorList(CustomerCompetitorQueryVO query) {
        return competitorMapper.selectList(query);
    }
}
